      program calgrid_to_v5d
c=======================================================================
c
c	calgrid_to_v5d.f
c
c	Version 1.
c	Level 980510
c
c	Mike Barna
c	Sunday, May 10, 1998
c
c-----------------------------------------------------------------------
c
c	DESCRIPTION:
c
c	This program converts a CONC.DAT data file (generated from 
c	EarthTech's air quality model CALGRID) into a format
c	suitable for the data visualization program VIS5D.
c	
c	For a description of the CALGRID and VIS5D variables, see the 
c	respective user manuals.
c
c	A skeleton version of the this program can be downloaded from the
c	Vis5d web page:
c
c	http://www.ssec.wisc.edu/~billh/vis5d.html
c
c-----------------------------------------------------------------------
c
c	REVISIONS:
c
c	From 1A to 1B:
c
c		1)  Change VERTICAL and PROJECTION arguments
c
c-----------------------------------------------------------------------
c
c	INSTRUCTIONS:
c
c	1)  Make sure you have the source code (CALGRID_TO_V5D.F), external
c		subprograms (BINIO.C, V5D.C), and "include" files (BINIO.H,
c		V5D.H, V5DF.H, VIS5D.H, and CALGRID.H) in the same directory.
c
c	2)  Run MAKE
c
c	3)  Execute CALGRID_TO_V5D.EXE
c
c	4)  Type the name of the CALGRID.DAT input file and the 
c		YOURFILE.V5D output file.
c
c-----------------------------------------------------------------------
c
c	EXAMPLE MAKEFILE:
c
c	# makefile for calgrid_to_v5d.f conversion program
c
c	calgrid_to_v5d: calgrid_to_v5d.o binio.o v5d.o 
c	f77 +e +U77 binio.o v5d.o calgrid_to_v5d.o -lm -o calgrid_to_v5d 
c
c	calgrid_to_v5d.o: calgrid_to_v5d.f
c	f77 +e +U77 -c calgrid_to_v5d.f	
c
c	binio.o:  binio.c
c	c89 -c binio.c
c
c	v5d.o:  v5d.c
c	c89 -c v5d.c
c
c-----------------------------------------------------------------------
c
c	COMMENTS FROM THE SKELETON PROGRAM:
c
c This is a skeleton conversion program for converting your data format
c to VIS-5D's v5d format (which is directly read by vis5d).
c
c This version allows you to specify a different number of levels per
c variable, map projections, and vertical coordinate systems.
c
c You need to insert code to do two things:
c     1. Open your input file and read the header information, putting
c        that information into the required variables.  If your file
c        doesn't contain some of the needed values you can just assign
c        constants to the variables.
c     2. read your grid data into the array G for each timestep and
c        variable.
c
c Pay special attention to comments in UPPERCASE.  They give more
c information about what you have to do.
c
c The rest of the program will handle creating the v5d file, writing the
c grid data, and closing the file.
c
c Compile this program with the foo2_to_v5d.f.m makefile.
c Run this program with two arguments:  the name of your input file and
c the name of the v5d output file.
c
c=======================================================================
c
      EXTERNAL v5dcreate,v5dclose
c
c --- Include parameter statements
      include 'calgrid.h'
      include 'v5df.h'
c
c --- Local vars (from foo2_to_v5d template)
      integer n
      real G(MAXROWS, MAXCOLUMNS, MAXLEVELS)
      character*100 fngrid,fnv5d,fnlog										
      integer maxnl
      integer i
c
c --- Calgrid header record variables:
	integer iogrid,nx,ny,nz,ibyr,ibjul,ibhr,irlg,nszout
	real dxkm,dykm
	character*15 csout(mxnsz)
c	
C  THE FOLLOWING VARIABLES DESCRIBE THE DATASET TO BE CONVERTED.  YOU
C  MUST INITIALIZE ALL THESE VARIABLES WITH VALUES FROM YOUR FILE OR
C  ASSIGN SUITABLE CONSTANTS.  SEE THE README FILE FOR DESCRIPTIONS
C  OF THESE VARIABLES.
      integer nr, nc, nl(MAXVARS)
      integer numtimes
      integer numvars
      character*10 varname(MAXVARS)
      integer dates(MAXTIMES)
      integer times(MAXTIMES)
      integer compressmode
      integer projection
      real proj_args(100)
      integer vertical
      real vert_args(MAXLEVELS)
c
	character*8 vernum,levnum
c
c --- DATA statement from foo2_to_v5d
c    initialize the variables to missing values
      data nr,nc / IMISSING, IMISSING /
      data (nl(i),i=1,MAXVARS) / MAXVARS*IMISSING /
      data numtimes,numvars / IMISSING, IMISSING /
      data (varname(i),i=1,MAXVARS) / MAXVARS*"          " /
      data (dates(i),i=1,MAXTIMES) / MAXTIMES*IMISSING /
      data (times(i),i=1,MAXTIMES) / MAXTIMES*IMISSING /
      data compressmode / 1 /
      data projection / IMISSING /
      data (proj_args(i),i=1,100) / 100*MISSING /
      data vertical / IMISSING /
      data (vert_args(i),i=1,100) / 100*MISSING /
c
c --- Data statement to define version, level:
	data vernum/'1B'/
	data levnum/'980510'/
c
c --- Print header to terminal, ask user to input file names:
	write(ioterm,*)
	write(ioterm,*) '***** calgrid_to_v5d *****'
	write(ioterm,*)
	write(ioterm,*) 'Mike Barna - WSU'
	write(ioterm,*) 'Version: ',vernum
	write(ioterm,*) 'Level: ',levnum
	write(ioterm,*)
	write(ioterm,*) 'This program will convert a CONC.DAT data'
	write(ioterm,*) 'file into a Vis5D input file.'
	write(ioterm,*) 
	write(ioterm,*) 'Please enter the name of the CONC.DAT file:'
	read(ioinp,*),fngrid
	write(ioterm,*) 
	write(ioterm,*) 'Please enter the name of the VIS5D file:'
	read(ioinp,*),fnv5d
	write(ioterm,*) 
	write(ioterm,*) 'Please enter the name of the LOG file:'
	read(ioinp,*),fnlog
	write(ioterm,*)
c
c --- Open LOG file:
      open(iolog,file=fnlog,status='new',form='formatted')
c
c --- Print header to log file:
	write(iolog,*)
	write(iolog,*) '***** calgrid_to_v5d *****'
	write(iolog,*)
	write(iolog,*) 'Mike Barna - WSU'
	write(iolog,*) 'Version: ',vernum
	write(iolog,*) 'Level: ',levnum
	write(iolog,*)
	write(iolog,*) 'CONC.DAT file name: ',fngrid
	write(iolog,*) 'VIS5D file name: ',fnv5d
	write(iolog,*) 'LOG file name: ',fnlog
	write(iolog,*)
c
c --- open CONC.DAT binary input file:
      open(iogrid,file=fngrid,status='old',form='unformatted')
c
c********************************
c --- READ CALGRID HEADER RECORDS:
c********************************
c
c ---	Read the first header record:
c
      call rwh1(iogrid,nx,ny,nz,ibyr,ibjul,ibhr,irlg,
     1          dxkm,dykm,nszout)
c
c ---	Read the second and third header records:
c
      call rwh23(iogrid,nszout,csout)
c
c************************************************
c --- ASSIGN CALGRID VALUES TO V5D VARIABLE NAMES:
c************************************************
c
c --- Remember, you need to assign:
c		NR,NC,NL(MAXVARS),NUMTIMES,NUMVARS,VARNAME(MAXVARS),
c		DATES(MAXTIMES),TIMES(MAXTIMES),COMPRESSMODE,PROJECTION,
c		PROJ_ARGS(100),VERTICAL,VERT_ARGS(MAXLEVELS)
c	See Vis5D user's manual for variable description...
c
	write(iolog,*) 'Assign CONC.DAT values to VIS5D variables:'
	write(iolog,*)
c
     	NUMTIMES=irlg
		write(iolog,*) 'NUMTIMES: ',numtimes
c
	NR=ny
		write(iolog,*) 'NR: ',nr
c
	NC=nx
		write(iolog,*) 'NC: ',nc
c
	VERTICAL=0
		write(iolog,*) 'VERTICAL: ',VERTICAL
	VERT_ARGS(1) = 1
		write(iolog,*) 'VERT_ARGS(1): ', VERT_ARGS(1)
	VERT_ARGS(2) = 1
		write(iolog,*) 'VERT_ARGS(2): ', VERT_ARGS(2)
c
      PROJECTION = 1
		write(iolog,*) 'PROJECTION: ',projection
      PROJ_ARGS(1) = 50.175
		write(iolog,*) 'PROJ_ARGS(1): ',proj_args(1)
      PROJ_ARGS(2) = 125.003
		write(iolog,*) 'PROJ_ARGS(2): ',proj_args(2)
      PROJ_ARGS(3) = 0.0450
		write(iolog,*) 'PROJ_ARGS(3): ',proj_args(3)
      PROJ_ARGS(4) = 0.0667
		write(iolog,*) 'PROJ_ARGS(4): ',proj_args(4)
c
      NUMVARS=nszout/nz
c
c --- set number of levels and name for each species
c
      do nv=1,numvars
         NL(nv) = nz
		write(iolog,*) 'NL(nv): ',nl(nv)
         VARNAME(nv) = csout(nv*nz)
		write(iolog,*) 'VARNAME(nv): ',varname(nv)
      enddo
c
c --- create dates and times for Vis5D
c
      ileap = 1-max(1,mod(ibyr,4))
      itime = 1
 100  DATES(itime) = 1000*ibyr+ibjul
      TIMES(itime) = 10000*ibhr
c
c --- advance by one hour (this will fail in going from 2300 99Dec31 to
c --- 0000 00Jan01)
c
      if(itime.lt.numtimes) then
         ibhr = ibhr+1
         if(ibhr.gt.23) then
            ibhr = 0
            ibjul = ibjul+1
            if(ibjul.gt.365+ileap) then
               ibjul = 1
               ibyr = ibyr+1
            endif
         endif
         itime = itime+1
         goto 100
      endif
c
c*************************
c --- CREATE THE V5D FILE:
c*************************
c
c     Create the v5d file.
      n = v5dcreate( fnv5d, numtimes, numvars, nr, nc, nl,
     *               varname, times, dates, compressmode,
     *               projection, proj_args, vertical, vert_args )
c
      if (n .eq. 0) then
         call exit(1)
      endif
c
c*************************************
c --- ENTER THE CONVERSION SUBROUTINE:
c*************************************
c
      maxnl = nl(1)
      do i=1,numvars
         if (nl(i) .gt. maxnl) then
            maxnl = nl(i)
         endif
      enddo
c
      call convert(nr,nc,nl,maxnl,numtimes,numvars,G,nx,ny,nz)
c
c*************************************
c --- YOU'RE DONE!
c*************************************
c
c     close the v5d file and exit
      n = v5dclose()
      if (n .eq. 0) then
c        failed
         call exit(1)
      else
c        success
         call exit(0)
      endif
C
	end
c
c***************************
c --- END OF MAIN PROGRAM...
c***************************
c
c----------------------------------------------------------------------
      subroutine convert(nr,nc,nl,maxnl,numtimes,numvars,G,nx,ny,nz)
c----------------------------------------------------------------------
c
c	Read in the calgrid data from conc.dat, and 
c		write the data in the proper vis5d format using the 
c		v5dwrite function.  
c
      include 'calgrid.h'
      include 'v5df.h'
c
c --- Arguments:
      integer nr
      integer nc
      integer nl(MAXVARS)
      integer maxnl
      integer numtimes,numvars
      real G(MAXROWS, MAXCOLUMNS, MAXLEVELS)
c
c --- Local variables:
      integer it, iv, n
c
c --- calgrid variables:
	integer nx,ny,nz
	integer nyr,njul,nhr
	real conc2d,conc3d
	character*15 clabz
c
	integer iii,jjj,kkk
c
	dimension conc2d(nx,ny),conc3d(nx,ny,nz)
c
C  READ YOUR DATA FOR TIME STEP NUMBER IT AND VARIABLE NUMBER IV INTO
C  THE ARRAY G HERE.  NOTE THAT G(1,1,1) IS THE NORTH WEST BOTTOM CORNER
C  AND G(NR,NC,NL) IS THE SOUTH EAST TOP CORNER.  ALSO, THE LOOPS CAN
C  BE CHANGED TO WORK IN ANY ORDER, THE v5dwrite CALL WILL STILL WORK.
C  VALUES GREATER THEN 1.0E30 ARE CONSIDERED TO BE 'MISSING'.
c
	print*,nr,nc,maxnl,numtimes,numvars,nx,ny,nz
c
      do it=1,numtimes
c
	  write(ioterm,*)'PROCESSING HOUR: ',it
	  write(iolog,*)'PROCESSING HOUR: ',it
c
        read(iogrid) nyr,njul,nhr
	print*,nyr,njul,nhr
c
          do iv=1,numvars
c
            do kkk=1,nz
		    read(iogrid) clabz,conc2d
			  do jjj=1,ny
			    do iii=1,nx
				  conc3d(iii,jjj,kkk)=conc2d(iii,jjj)
   			    enddo
		      enddo
	      enddo
c
	      call convertR3(it,iv,nr,nc,nl(iv),G,conc3d,nx,ny,nz)
c
         enddo
      enddo
c
      return
c
      end
c
c-----------------------------------------------------------------------
      subroutine convertR3(it,iv,nr,nc,nlevel,G,real3d,nx,ny,nz)
c-----------------------------------------------------------------------
c
      EXTERNAL v5dwrite
      include 'calgrid.h'
      include 'v5df.h'
      integer it,iv,nr,nc,nlevel
	real G,real3d
	dimension G(nr,nc,nlevel),real3d(nx,ny,nz)
      do ir = 1,nr
         jr = nr-ir+1
         do ic = 1,nc
            do k = 1,nlevel
               G(ir,ic,k) = real3d(ic,jr,k)
            enddo
         enddo
      enddo
c
      n = v5dwrite( IT, IV, G )
      if (n .eq. 0) then
c        error
         print *,'Error return from w5dwrite in convertR3'
         call exit(1)
      endif
      return
c
      end
c
c=======================================================================
	subroutine rwh1(iogrid,nx,ny,nz,ibyr,ibjul,ibhr,irlg,
     .          dxkm,dykm,nszout)					 
c=======================================================================
c
c	SUBROUTINE RWH1
c
c	PURPOSE:
c
c		Read the first header record in conc.dat
c
c	VARIABLES:
c
c	cmodel	char*12		Model name
c	ver		char*12		Model version number
c	level	char*12		Model level number
c	ibyr	integer		Starting year of run
c	ibjul	integer		Starting Julian day
c	ibhr	integer		Starting hour (0-23)
c	irlg	integer		Length of run (hours)
c	iavg	integer		Averaging time of output (hours)
c	nxm		integer		Number of grid points in Calmet grid (x-dir)	
c	nym		integer		Number of grid points in Calgrid grid (y-dir)
c	nx		integer		Number of last Calgrid grid (x-dir)
c	ny		integer		Number of last Calgrid grid (y-dir)		
c	nz		integer		Number of last Calgrid grid (z-dir)
c	ione	integer		Number of first Calgrid grid (must be "1")
c	izero	integer		Not used by Calgrid
c	nszout	integer		Number of species-layer combinations stored
c							in conc.dat
c	dxkm	real		Grid spacing (x-dir)
c	dykm	real		Grid spacing (y-dir)
c	lsgrid	logical		Sampling grid flag (T=gridded receptors, F=
c							no gridded receptors)
c
c----------------------------------------------------------------------
c
c --- Declare header record variables:
c     
	character*12 cmodel,ver,level
	integer ibyr,ibjul,ibhr,irlg,iavg,nxm,nym,nx,ny,nz,ione,
	1	izero,nszout
	real dxkm,dykm
	logical lsgrid
c
c  Read first header record
c
	read(iogrid)cmodel,ver,level,ibyr,ibjul,ibhr,
     1 irlg,iavg,nxm,nym,dxkm,dykm,nz,ione,nx,
     1 ione,ny,ione,nx,ione,ny,ione,izero,izero,
     1 izero,izero,lsgrid,nszout
c
	return
	end
c	
c=======================================================================
	 subroutine rwh23(iogrid,nszout,csout)
c=======================================================================
c
c	SUBROUTINE RWH23
c
c	PURPOSE:
c
c		Read the second and third header records of conc.dat
c
c	VARIABLES:
c
c	title(3)		char*80		User specified run title
c	csout(nszout)	char*15		Species name (first 11 characters) and 
c								layer (last 3 characters) of concen-
c								trations stored in conc.dat
c----------------------------------------------------------------------
c
c --- Declare header record variables:
c     
	character*80 title(3)
	character*15 csout(nszout)
c
c --- Read the run title (header #2)
c
	read(iogrid) title
c
c --- Read list of species layers
c
      read(iogrid) csout
c
	return
	end
